/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Tracy Norris (tnorris@nc.rr.com), Jesper Kristensen (jesper@nc.rr.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
 // This holds the object for accessing the preferences
var autofillPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService).getBranch("autofill.");

setInternalPrefs();

window.addEventListener('load', doVisualize, true);
window.addEventListener('load', setButtonStatus, true);

// preference observer
var prefChangeObserver = {
  observe :

  function(aSubject, aTopic, aData) {
	 setButtonStatus();
  }
};

// add observer to preference
var qi = autofillPrefs.QueryInterface(Components.interfaces["nsIPrefBranchInternal"]);
qi.addObserver("", prefChangeObserver, false);


// Set Button Status based on configuration
function setButtonStatus() {
	consoleDump("In setButtonStatus");

	// call doVisualize to turn the button on/off based on page content
	doVisualize();
	
	var autofillButton = document.getElementById("autofill-button");
	if (autofillPrefs.prefHasUserValue("profile_list")) {
		var profileList=autofillPrefs.getCharPref("profile_list");

		// if there are no profiles, disable the button (regardless of page content)
		if (profileList.length==0) {
			consoleDump("Disabling Attribute (no profiles)");
			autofillButton.setAttribute("disabled",true);
			autofillButton.removeAttribute("context");
		}
		// if there is only one profile, keep the status of the button, but make it not a menu button (and disable context)
		else if (profileList.indexOf(",")==-1) {
			consoleDump("UnMenuing Button");
			autofillButton.removeAttribute("type");
			autofillButton.removeAttribute("context");
		}
		// If there is more than one profile, leave the status of the button, but make it a menu button
		else {
			consoleDump("More than one profile, making menu-button");
			autofillButton.setAttribute("type","menu-button");
		}
	}
	else {
		consoleDump("Disabling Attribute (no config)");
		autofillButton.setAttribute("disabled",true);
		autofillButton.removeAttribute("type");
		autofillButton.removeAttribute("context");
	}
}

function populateAutofillMenu(event) {
	if (autofillPrefs.prefHasUserValue("selected_profile_id"))
		selectedProfileId=autofillPrefs.getCharPref("selected_profile_id");
	else
		selectedProfileId="";
		
	
	clearMenu(event.target);
	
	if (autofillPrefs.prefHasUserValue("profile_list"))
		profileList=autofillPrefs.getCharPref("profile_list");
	else
		profileList="";

	var profiles=profileList.split(",");

	// Only show the list of profiles if there is more than 1
	if (profiles.length>1) {
		for (i=0;i<profiles.length;++i) {
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label",profiles[i]+" "); // Add a space at the end to prevent theme conflicts (like profiles named "home")
			menuitem.setAttribute("index",i);
			menuitem.setAttribute("id",profiles[i]);
			menuitem.setAttribute("type","radio");
			menuitem.setAttribute("name","profiles");

			if (profiles[i]==selectedProfileId) {
				menuitem.setAttribute("checked",true);
			}

			event.target.appendChild(menuitem);
		}

		consoleDump("Profiles length: " + profiles.length);
	}

	consoleDump("onPopulate: " + event.target.id + ":" + event.target.getAttribute("showOptions"));
	
	if (event.target.getAttribute("showOptions")=="true") { // ToDo: is there a way to return a custom attribute as a boolean?
		if (profiles.length<=1)
			optionsIndex=0;
		else
			optionsIndex=profiles.length+1;

		if (optionsIndex>0) {
			var menuSeparator=document.createElement("menuseparator");
			menuSeparator.setAttribute("index",profiles.length);
			event.target.appendChild(menuSeparator);
		}

		var menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label","Autofill Options");
		menuitem.setAttribute("index",optionsIndex);
		menuitem.setAttribute("id","autofill-menu-options");
		menuitem.setAttribute("oncommand","showOptions();event.preventBubble();");
		event.target.appendChild(menuitem);
	}
	
	return true;
}

function showOptions() {
	window.openDialog("chrome://autofill/content/autofill_pref_dialog.xul","autofill_pref_dialog","chrome");
}


function doProfilePick(event) {
	consoleDump("Pick Profile:" + event.target.id);
	var selectedProfileId=autofillPrefs.setCharPref("selected_profile_id", event.target.id);
	return true;
}


function clearMenu(aParent) {
  var children = aParent.childNodes;
  for (var i = 0; i < children.length; i++) {
      var index = children[i].getAttribute("index");
      if (index)
        aParent.removeChild(children[i]);
   }
}


function doAutofill(event) {
	processFields(true);
}


function doVisualize() {
	
	if (autofillPrefs.prefHasUserValue("profile_list")) {			
		processFields(false);			
		if (autofillPrefs.getBoolPref("autohighlight") == true) {
			var linkElm = window.content.document.createElement("link");
			linkElm.setAttribute("id", "autofillVisualize");
			linkElm.setAttribute("href", "chrome://autofill/skin/visualize.css");
			linkElm.setAttribute("rel", "stylesheet");
			linkElm.setAttribute("type", "text/css");
			
			window.content.document.documentElement.childNodes[0].appendChild(linkElm);		
		}
	}
	else {
		document.getElementById("autofill-button").setAttribute("disabled","true");		
	}
}

function processFields(bChange) {

	var autofillPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autofill.");

	var formFields = new Array("title","first_name","middle_name","last_name","suffix","email_address","phone_number","primary_line1",
				"primary_line2","primary_city","primary_state","primary_zip","primary_country","secondary_line1","secondary_line2",
				"secondary_city","secondary_state","secondary_zip","secondary_country","primary_country_code","secondary_country_code");

	var profiles=autofillPrefs.getCharPref("profile_list");
	
	var enableAutofill = false;

	if (profiles.length>0) {
		var profileList=profiles.split(",");

		// select correct profile
		var profilenumber=0;
		for (var i=0;i<profileList.length;i++) {
			if (autofillPrefs.getCharPref("selected_profile_id") == profileList[i]) {
				profilenumber = i;
			}
		}

		var formValues = {};
		for (var i=0;i<formFields.length;i++) {
			formValues[formFields[i]]=autofillPrefs.getCharPref(profileList[profilenumber]+"."+formFields[i]);
		}
		
		// Copy primary address values to secondary if required
		if (autofillPrefs.getBoolPref(profileList[profilenumber]+".use_primary") == true) {
			formValues["secondary_city"] = formValues["primary_city"];
			formValues["secondary_country"] = formValues["primary_country"];
			formValues["secondary_country_code"] = formValues["primary_country_code"];
			formValues["secondary_line1"] = formValues["primary_line1"];
			formValues["secondary_line2"] = formValues["primary_line2"];
			formValues["secondary_state"] = formValues["primary_state"];
			formValues["secondary_zip"] = formValues["primary_zip"];
		}
		
		// Setup fullname
		formValues["full_name"] = formValues["first_name"];
		if (formValues["middle_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["middle_name"];
		}
		if (formValues["last_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["last_name"];
		}
		if (formValues["suffix"] != "") {
			formValues["full_name"] = formValues["full_name"] + ", " + formValues["suffix"];
		}
		formFields[formFields.length] = "full_name";

		// check for frames
		var myFrames = window.content.document.getElementsByTagName("frame");
		if(myFrames.length == 0) {
			// no frames, get forms
			var pageForms = window.content.document.getElementsByTagName("form");
		}
		else {
			var pageForms = new Array();
			var formCount = 0;
			for(var fr = 0; fr < myFrames.length; fr++) {
				myForms = myFrames[fr].contentDocument.getElementsByTagName("form");
				for(var fo = 0; fo < myForms.length; fo++) {
					pageForms[formCount++] = myForms[fo];
				}
			}
		}

		// loop through all forms
		for (var i = 0; i < pageForms.length; i++)	{
			var formElements = pageForms[i].elements;
			// loop through all elememts
			for (var j = 0; j < formElements.length; j++) {
				// get attributes
				var elmType = formElements[j].type.toLowerCase();
				var elmName = formElements[j].name.toLowerCase();
                             			
				if (elmType == "text" || elmType == "textarea" || elmType == "password") {					
					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = autofillPrefs.getCharPref("form_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (elmName == fieldnames[k].toLowerCase()) {
									if (bChange) {
										formElements[j].value =	formValues[formFields[l]];
									}								
									else {
										formElements[j].setAttribute("autofillsupport","true");
										enableAutofill = true;
									}
								}
							}
						}			
					}					
				}
				else if(elmType == "select-one" || elmType == "select-multiple") {
					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = autofillPrefs.getCharPref("form_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (elmName == fieldnames[k].toLowerCase()) {
									if (bChange) {
										var selElement = formElements[j];
										for (var c=0; c<selElement.length;c++ ) {
											if (selElement[c].value == formValues[formFields[l]]) {
												selElement[c].selected = true;
											}
										}			                        
									}								
									else {
										formElements[j].setAttribute("autofillsupport","true");
										enableAutofill = true;
									}
								}
							}
						}			
					}					
				}
			}
		}
	}
	
	if (!bChange) {
		if (enableAutofill) {
			document.getElementById('autofill-button').removeAttribute("disabled");			
			document.getElementById('autofill-button').setAttribute("context","autofill-menu");			
		}
		else {
			document.getElementById('autofill-button').setAttribute("disabled","true");	
			document.getElementById('autofill-button').removeAttribute("context");	
		}
	}
}


